/**
 *  Copyright (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  Licensed under the GPL, Version 3.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *       http://www.gnu.org/licenses/gpl.html
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 * --------------------------------------------------------------------------
 *  版权 (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  本开源软件遵循 GPL 3.0 协议;
 *  如果您不遵循此协议，则不被允许使用此文件。
 *  你可以从下面的地址获取完整的协议文本
 *
 *       http://www.gnu.org/licenses/gpl.html
 */
package org.tinygroup.flowplugin;

import java.util.Iterator;
import java.util.Map;
import java.util.Map.Entry;

import org.tinygroup.cepcore.CEPCore;
import org.tinygroup.flow.FlowExecutor;
import org.tinygroup.logger.LogLevel;
import org.tinygroup.logger.Logger;
import org.tinygroup.logger.LoggerFactory;
import org.tinygroup.plugin.config.PluginConfiguration;
import org.tinygroup.plugin.config.PluginService;
import org.tinygroup.plugin.impl.BasePlugin;
import org.tinygroup.springutil.SpringUtil;

public class FlowPlugin extends BasePlugin {
	/**
	 * 通过依赖关系获取的cepcore实例
	 */
	private CEPCore cepcore;
	private FlowPluginProcessor flowProcessor;
	private static Logger logger = LoggerFactory.getLogger(FlowPlugin.class);

	// private String initResourcePath = null;
	// private ClassLoader initLoader;

	public void init() {
		super.init();
		initProcessors();
	}

	protected ClassLoader getThisClassLoader() {
		return FlowPlugin.class.getClassLoader();
	}

	protected void initServiceInstance(PluginService service) {
		try {
			String className = service.getClassName();
			Class<?> clazz = getThisClassLoader().loadClass(className);
			Object bean = null;
			if (service.getInstanceGetMode() == null
					|| service.getInstanceGetMode().equalsIgnoreCase("new")) {
				bean = clazz.newInstance();
			} else {
				bean = SpringUtil.getBean(service.getId());
				if (bean == null) {
					bean = SpringUtil.getBean(clazz);
				}
			}
			if (bean != null) {
				getServiceInstanceMap().put(service, bean);
			} else {
				logger.logMessage(LogLevel.ERROR,
						"服务[id:{0}，version:{1}]的实例获取失败，请检查配置！",
						service.getId(), service.getVersion());
			}
		} catch (Exception e) {
			logger.errorMessage("初始化插件服务[id:{0},version:{1}]出错,", e,
					service.getId(), service.getVersion());
		}
	}

	public void initProcessors() {

		logger.logMessage(LogLevel.DEBUG, "初始化FlowPluginProcessor列表");

		flowProcessor = new FlowPluginProcessor();
		Map<PluginService, Object> map = getAllServiceMap();
		Iterator<Entry<PluginService, Object>> iterator = map.entrySet()
				.iterator();
		while (iterator.hasNext()) {
			Entry<PluginService, Object> entry = iterator.next();
			dealExecutor(entry, flowProcessor);
		}
		logger.logMessage(LogLevel.DEBUG, "初始化FlowPluginProcessor列表完成");
	}

	/**
	 * 根据配置信息，对每个插件服务进行处理
	 * 
	 * @param entry
	 */
	private void dealExecutor(Entry<PluginService, Object> entry,
			FlowPluginProcessor flowProcessor) {
		PluginService service = entry.getKey();
		FlowExecutor executor = (FlowExecutor) entry.getValue();
		logger.logMessage(LogLevel.DEBUG,
				"处理FlowPlugin的Service[id:{0},version:{1}]", service.getId(),
				service.getVersion());

		flowProcessor.addExecutor(executor);

		logger.logMessage(LogLevel.DEBUG,
				"处理FlowPlugin的Service[id:{0},version:{1}]完成", service.getId(),
				service.getVersion());
	}

	public void start() {
		logger.logMessage(LogLevel.DEBUG, "启动FlowPlugin");
		cepcore.registerEventProcessor(flowProcessor);
		logger.logMessage(LogLevel.DEBUG, "启动FlowPlugin完成");
	}

	public void pause() {
		logger.logMessage(LogLevel.DEBUG, "暂停FlowPlugin");
		cepcore.unregisterEventProcessor(flowProcessor);
		logger.logMessage(LogLevel.DEBUG, "暂停FlowPlugin完成");
	}

	public void stop() {
		logger.logMessage(LogLevel.DEBUG, "停止FlowPlugin");
		cepcore.unregisterEventProcessor(flowProcessor);
		logger.logMessage(LogLevel.DEBUG, "停止FlowPlugin完成");
	}

	public void destroy() {
		logger.logMessage(LogLevel.DEBUG, "销毁FlowPlugin");
		super.destroy();
		// removeInitResourcePath();
		logger.logMessage(LogLevel.DEBUG, "销毁FlowPlugin完成");
	}

	public <T> void setService(T service, Class<T> clazz) {
		if (CEPCore.class.isAssignableFrom(clazz)) {
			if (service == null) {
				this.cepcore = null;
			} else {
				this.cepcore = (CEPCore) service;
			}
		}

	}

	public <T> void setConfig(PluginConfiguration pluginConfiguration) {
	
	}

}
